@@ -45,6 +45,8 @@ import sys
 import time
 import uuid
 
+import codegen
+
 import xml.etree.ElementTree as et
 import xml.sax.saxutils as saxutils
 
@@ -54,6 +56,55 @@ import oa_guid_registry as oa_registry
 # MDAPI configs include writes to some non-config registers,
 # thus the blacklists...
 
+hsw_chipset_params = {
+    'a_offset': 12,
+    'b_offset': 192,
+    'c_offset': 224,
+    'oa_report_size': 256,
+    'registers': {
+        # TODO extend the symbol table for nicer output...
+        0x2710: { 'name': 'OASTARTTRIG1' },
+        0x2714: { 'name': 'OASTARTTRIG1' },
+        0x2718: { 'name': 'OASTARTTRIG1' },
+        0x271c: { 'name': 'OASTARTTRIG1' },
+        0x2720: { 'name': 'OASTARTTRIG1' },
+        0x2724: { 'name': 'OASTARTTRIG6' },
+        0x2728: { 'name': 'OASTARTTRIG7' },
+        0x272c: { 'name': 'OASTARTTRIG8' },
+        0x2740: { 'name': 'OAREPORTTRIG1' },
+        0x2744: { 'name': 'OAREPORTTRIG2' },
+        0x2748: { 'name': 'OAREPORTTRIG3' },
+        0x274c: { 'name': 'OAREPORTTRIG4' },
+        0x2750: { 'name': 'OAREPORTTRIG5' },
+        0x2754: { 'name': 'OAREPORTTRIG6' },
+        0x2758: { 'name': 'OAREPORTTRIG7' },
+        0x275c: { 'name': 'OAREPORTTRIG8' },
+        0x2770: { 'name': 'OACEC0_0' },
+        0x2774: { 'name': 'OACEC0_1' },
+        0x2778: { 'name': 'OACEC1_0' },
+        0x277c: { 'name': 'OACEC1_1' },
+        0x2780: { 'name': 'OACEC2_0' },
+        0x2784: { 'name': 'OACEC2_1' },
+        0x2788: { 'name': 'OACEC3_0' },
+        0x278c: { 'name': 'OACEC3_1' },
+        0x2790: { 'name': 'OACEC4_0' },
+        0x2794: { 'name': 'OACEC4_1' },
+        0x2798: { 'name': 'OACEC5_0' },
+        0x279c: { 'name': 'OACEC5_1' },
+        0x27a0: { 'name': 'OACEC6_0' },
+        0x27a4: { 'name': 'OACEC6_1' },
+        0x27a8: { 'name': 'OACEC7_0' },
+        0x27ac: { 'name': 'OACEC7_1' },
+    },
+    'config_reg_blacklist': {
+        0x2364, # OASTATUS1 register
+    },
+    'register_offsets': {
+        0x1f0: 'PERFCNT 0',
+        0x1f8: 'PERFCNT 1',
+    },
+}
+
 gen8_11_chipset_params = {
     'a_offset': 16,
     'b_offset': 192,
@@ -62,74 +113,48 @@ gen8_11_chipset_params = {
     'config_reg_blacklist': {
         0x2364, # OACTXID
     },
+    'register_offsets': {
+        0x1f0: 'PERFCNT 0',
+        0x1f8: 'PERFCNT 1',
+    }
+}
+
+xehpsdv_chipset_params = {
+    'a_offset': 16,
+    'b_offset': 192,
+    'c_offset': 224,
+    'oa_report_size': 256,
+    'config_reg_blacklist': {
+        0x2364, # OACTXID
+    },
+    'register_offsets': {
+        0x1b0: 'PERFCNT 0',
+        0x1b8: 'PERFCNT 1',
+    }
 }
 
 chipsets = {
-    'HSW': {
-        'a_offset': 12,
-        'b_offset': 192,
-        'c_offset': 224,
-        'oa_report_size': 256,
-        'registers': {
-            # TODO extend the symbol table for nicer output...
-                0x2710: { 'name': 'OASTARTTRIG1' },
-            0x2714: { 'name': 'OASTARTTRIG1' },
-            0x2718: { 'name': 'OASTARTTRIG1' },
-            0x271c: { 'name': 'OASTARTTRIG1' },
-            0x2720: { 'name': 'OASTARTTRIG1' },
-            0x2724: { 'name': 'OASTARTTRIG6' },
-            0x2728: { 'name': 'OASTARTTRIG7' },
-            0x272c: { 'name': 'OASTARTTRIG8' },
-            0x2740: { 'name': 'OAREPORTTRIG1' },
-            0x2744: { 'name': 'OAREPORTTRIG2' },
-            0x2748: { 'name': 'OAREPORTTRIG3' },
-            0x274c: { 'name': 'OAREPORTTRIG4' },
-            0x2750: { 'name': 'OAREPORTTRIG5' },
-            0x2754: { 'name': 'OAREPORTTRIG6' },
-            0x2758: { 'name': 'OAREPORTTRIG7' },
-            0x275c: { 'name': 'OAREPORTTRIG8' },
-            0x2770: { 'name': 'OACEC0_0' },
-            0x2774: { 'name': 'OACEC0_1' },
-            0x2778: { 'name': 'OACEC1_0' },
-            0x277c: { 'name': 'OACEC1_1' },
-            0x2780: { 'name': 'OACEC2_0' },
-            0x2784: { 'name': 'OACEC2_1' },
-            0x2788: { 'name': 'OACEC3_0' },
-            0x278c: { 'name': 'OACEC3_1' },
-            0x2790: { 'name': 'OACEC4_0' },
-            0x2794: { 'name': 'OACEC4_1' },
-            0x2798: { 'name': 'OACEC5_0' },
-            0x279c: { 'name': 'OACEC5_1' },
-            0x27a0: { 'name': 'OACEC6_0' },
-            0x27a4: { 'name': 'OACEC6_1' },
-            0x27a8: { 'name': 'OACEC7_0' },
-            0x27ac: { 'name': 'OACEC7_1' },
-        },
-        'config_reg_blacklist': {
-            0x2364, # OASTATUS1 register
-        },
-    },
+    'HSW': hsw_chipset_params,
     'BDW': gen8_11_chipset_params,
     'CHV': gen8_11_chipset_params,
-    'SKLGT2': gen8_11_chipset_params,
-    'SKLGT3': gen8_11_chipset_params,
-    'SKLGT4': gen8_11_chipset_params,
+    'SKL': gen8_11_chipset_params,
     'BXT': gen8_11_chipset_params,
-    'KBLGT2': gen8_11_chipset_params,
-    'KBLGT3': gen8_11_chipset_params,
+    'KBL': gen8_11_chipset_params,
     'GLK': gen8_11_chipset_params,
-    'CFLGT2': gen8_11_chipset_params,
-    'CFLGT3': gen8_11_chipset_params,
+    'CFL': gen8_11_chipset_params,
     'CNL': gen8_11_chipset_params,
     'ICL': gen8_11_chipset_params,
     'EHL': gen8_11_chipset_params,
-    'TGLGT1': gen8_11_chipset_params,
-    'TGLGT2': gen8_11_chipset_params,
+    'TGL': gen8_11_chipset_params,
     'RKL': gen8_11_chipset_params,
     'DG1': gen8_11_chipset_params,
     'ADL': gen8_11_chipset_params,
+    'ACM': xehpsdv_chipset_params,
+    'MTL': xehpsdv_chipset_params,
 }
 
+xehp_plus = ( 'ACM', 'MTL' )
+
 register_types = { 'OA', 'NOA', 'FLEX', 'PM' }
 
 default_set_blacklist = { "RenderDX1x", # TODO: rename to something non 'DX'
@@ -145,35 +170,19 @@ counter_blacklist = {
                          # investiguate how to get this value.
 }
 
-sys_vars = { "EuCoresTotalCount",
-             "EuSlicesTotalCount",
-             "SamplersTotalCount",
-             "EuThreadsCount",
-             "GpuMinFrequencyMHz",
-             "GpuMaxFrequencyMHz",
-             "GpuTimestampFrequency",
-             "SliceMask",
-             "SubsliceMask",
-             "EuSubslicesTotalCount"
-           }
-
 def underscore(name):
     s = re.sub('MHz', 'Mhz', name)
     s = re.sub('\.', '_', s)
     s = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', s)
+    s = re.sub('#', '_', s)
     return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s).lower()
 
 def print_err(*args):
     sys.stderr.write(' '.join(map(str,args)) + '\n')
 
-read_register_offsets = {
-    0x1f0: 'PERFCNT 0',
-    0x1f8: 'PERFCNT 1',
-}
-
 def read_value(chipset, offset):
-    if offset in read_register_offsets:
-        return read_register_offsets[offset]
+    if offset in chipsets[chipset]['register_offsets']:
+        return chipsets[chipset]['register_offsets'][offset]
     print_err("Unknown offset register at offset {0}".format(offset))
     assert 0
 
@@ -190,6 +199,7 @@ def read_token_to_rpn_read(chipset, token, raw_offsets):
     offset = int(offset_str, 16)
 
     if raw_offsets:
+        # Location in the HW reports
         a_offset = chipsets[chipset]['a_offset']
         b_offset = chipsets[chipset]['b_offset']
         c_offset = chipsets[chipset]['c_offset']
@@ -204,7 +214,15 @@ def read_token_to_rpn_read(chipset, token, raw_offsets):
             else:
                 assert 0
         elif offset < b_offset:
-            return "A " + str(int((offset - a_offset) / 4)) + " READ"
+            a_cnt_offset = int((offset - a_offset) / 4)
+            if chipset in xehp_plus:
+                # Most A counters are in a contiguous array, except
+                # this A37.
+                if a_cnt_offset == 42:
+                    return "A 37 READ"
+                return "A " + str(a_cnt_offset) + " READ"
+            else:
+                return "A " + str(a_cnt_offset) + " READ"
         elif offset < c_offset:
             return "B " + str(int((offset - b_offset) / 4)) + " READ"
         elif offset < report_size:
@@ -212,6 +230,7 @@ def read_token_to_rpn_read(chipset, token, raw_offsets):
         else:
             return "{0} READ".format(read_value(chipset, offset))
     else:
+        # Location in the accumulated deltas
         idx = int(offset / 8)
         if chipset == "HSW":
             # On Haswell accumulated counters are assumed to start
@@ -227,6 +246,23 @@ def read_token_to_rpn_read(chipset, token, raw_offsets):
                 return "C " + str(idx - 54) + " READ"
             else:
                 return "{0} READ".format(read_value(chipset, offset))
+        elif chipset in xehp_plus:
+            # For XEHPSDV the array of accumulated counters is
+            # assumed to start with a GPU_TIME then GPU_CLOCK,
+            # then 38 A counters, then 8 B counters and finally
+            # 8 C counters.
+            if idx == 0:
+                return "GPU_TIME 0 READ"
+            elif idx == 1:
+                return "GPU_CLOCK 0 READ"
+            elif idx < 40:
+                return "A " + str(idx - 2) + " READ"
+            elif idx < 48:
+                return "B " + str(idx - 40) + " READ"
+            elif idx < 56:
+                return "C " + str(idx - 48) + " READ"
+            else:
+                return "{0} READ".format(read_value(chipset, offset))
         else:
             # For Gen8+ the array of accumulated counters is
             # assumed to start with a GPU_TIME then GPU_CLOCK,
@@ -318,10 +354,38 @@ def strip_dx_apis(text):
 
     return stripped.strip()
 
+def add_gpu_core_clocks_if_missing(metric_set, counters, counter_deps):
+    if len(counters) < 1:
+        return
+
+    for name,element in counters.items():
+        if name == 'GpuCoreClocks':
+            return
+
+    print_err("WARNING: add missing GpuCoreClocks counter for MetricSets=\"{0}\"".format(metric_set.get('ShortName')))
+    counter = et.SubElement(metric_set, 'Metric')
+    counter.set("SymbolName", "GpuCoreClocks")
+    counter.set("SignalName", "oa.fixed")
+    counter.set("ShortName", "GPU Core Clocks")
+    counter.set("LongName", "The total number of GPU core clocks elapsed during the measurement.")
+    counter.set("Group", "GPU")
+    counter.set("UsageFlags", "Tier1 Frame Batch Draw")
+    counter.set("MetricType", "EVENT")
+    counter.set("ResultType", "UINT64")
+    counter.set("MetricUnits", "cycles")
+    counter.set("HWUnitType", "GPU")
+    counter.set("SnapshotReportReadEquation", "dw@0x0c")
+    counter.set("SnapshotReportDeltaFunction", "DELTA 32")
+    counter.set("DeltaReportReadEquation", "qw@0x08")
+    counter.set("NormalizationEquation", "")
+
+    counters["GpuCoreClocks"] = counter
+    counter_deps["GpuCoreClocks"] = []
+
 # For recursively appending counters in order of dependencies...
 def append_deps_and_counter(mdapi_counter, mdapi_counters, deps,
                             sorted_array, sorted_set):
-    symbol_name = mdapi_counter.get('SymbolName')
+    symbol_name = oa_registry.Registry.sanitize_symbol_name(mdapi_counter.get('SymbolName'))
 
     if symbol_name in sorted_set:
         return
@@ -526,6 +590,7 @@ guids = {}
 guids_xml = et.parse(args.guids)
 for guid in guids_xml.findall(".//guid"):
     hashing_key = oa_registry.Registry.chipset_derive_hash(guid.get('chipset'),
+                                                           guid.get('name'),
                                                            guid.get('mdapi_config_hash'))
     guids[hashing_key] = guid.get('id')
 
@@ -533,6 +598,14 @@ for arg in args.xml:
     mdapi = et.parse(arg)
 
     concurrent_group = mdapi.find(".//ConcurrentGroup")
+    chipset = oa_registry.Registry.chipset_name(concurrent_group.get('SupportedHW'))
+
+    chipset_fullname = chipset
+    if concurrent_group.get('SupportedGT') != None:
+        chipset_fullname = chipset_fullname + oa_registry.Registry.gt_name(concurrent_group.get('SupportedGT'))
+    if chipset not in chipsets:
+        print_err("WARNING: unsupported chipset {0}, consider updating {1}".format(chipset, __file__))
+        continue
 
     for mdapi_set in mdapi.findall(".//MetricSet"):
 
@@ -540,19 +613,12 @@ for arg in args.xml:
         if "OGL" not in apis and "OCL" not in apis and "MEDIA" not in apis:
             continue
 
-        set_symbol_name = mdapi_set.get('SymbolName')
+        set_symbol_name = oa_registry.Registry.sanitize_symbol_name(mdapi_set.get('SymbolName'))
 
         if set_symbol_name in sets:
             print_err("WARNING: duplicate set named \"" + set_symbol_name + "\" (SKIPPING)")
             continue
 
-        chipset = mdapi_set.get('SupportedHW')
-        if concurrent_group.get('SupportedGT') != None:
-            chipset = chipset + concurrent_group.get('SupportedGT')
-        if chipset not in chipsets:
-            print_err("WARNING: unsupported chipset {0}, consider updating {1}".format(chipset, __file__))
-            continue
-
         if args.whitelist:
             set_whitelist = args.whitelist.split()
             if set_symbol_name not in set_whitelist:
@@ -571,7 +637,7 @@ for arg in args.xml:
 
         set = et.SubElement(metrics, 'set')
 
-        set.set('chipset', chipset)
+        set.set('chipset', chipset_fullname)
 
         set.set('name', mdapi_set.get('ShortName'))
         set.set('symbol_name', set_symbol_name)
@@ -628,16 +694,17 @@ for arg in args.xml:
             add_register_config(set, 0, None, flex_regs, "FLEX")
 
         mdapi_hw_config_hash = oa_registry.Registry.mdapi_hw_config_hash(mdapi_set)
-        guid_hash = oa_registry.Registry.chipset_derive_hash(chipset.lower(),
+        guid_hash = oa_registry.Registry.chipset_derive_hash(chipset_fullname.lower(),
+                                                             set_symbol_name,
                                                              mdapi_hw_config_hash)
         hw_config_hash = oa_registry.Registry.hw_config_hash(set)
 
         if guid_hash in guids:
             set.set('hw_config_guid', guids[guid_hash])
         else:
-            print_err("WARNING: No GUID found for metric set " + chipset + ", " + set_symbol_name + " (SKIPPING)")
+            print_err("WARNING: No GUID found for metric set " + chipset_fullname + ", " + set_symbol_name + " (SKIPPING)")
             print_err("WARNING: If this is a new config add the following to guids.xml:")
-            print_err("<guid config_hash=\"" + hw_config_hash + "\" mdapi_config_hash=\"" + mdapi_hw_config_hash + "\" id=\"" + str(uuid.uuid4()) + "\" chipset=\"" + chipset.lower() + "\" name=\"" + set_symbol_name + "\" />")
+            print_err("<guid config_hash=\"" + hw_config_hash + "\" mdapi_config_hash=\"" + mdapi_hw_config_hash + "\" id=\"" + str(uuid.uuid4()) + "\" chipset=\"" + chipset_fullname.lower() + "\" name=\"" + set_symbol_name + "\" />")
             metrics.remove(set)
             continue
 
@@ -656,7 +723,7 @@ for arg in args.xml:
         mdapi_counter_deps = {}
 
         for mdapi_counter in mdapi_set.findall("Metrics/Metric"):
-            symbol_name = mdapi_counter.get('SymbolName')
+            symbol_name = oa_registry.Registry.sanitize_symbol_name(mdapi_counter.get('SymbolName'))
 
             if symbol_name in counter_blacklist:
                 continue;
@@ -675,12 +742,13 @@ for arg in args.xml:
             equations = expand_macros(equations)
             equations = equations.replace('$$', "$")
             for token in equations.split():
-                if token[0] == '$' and token[1:] not in sys_vars and token[1:] != "Self":
+                if token[0] == '$' and not codegen.is_hw_var(token) and token[1:] != "Self":
                     deps.append(token[1:])
 
             mdapi_counters[symbol_name] = mdapi_counter
             mdapi_counter_deps[symbol_name] = deps
 
+        add_gpu_core_clocks_if_missing(mdapi_set, mdapi_counters, mdapi_counter_deps)
         sorted_mdapi_counters = sort_counters(mdapi_counters, mdapi_counter_deps)
 
         for mdapi_counter in sorted_mdapi_counters:
@@ -709,11 +777,11 @@ for arg in args.xml:
                 #mdapi_counter.set('DeltaReportReadEquation', '$GpuCoreClocks $GpuTime UDIV')
 
 
-            symbol_name = mdapi_counter.get('SymbolName')
+            symbol_name = oa_registry.Registry.sanitize_symbol_name(mdapi_counter.get('SymbolName'))
 
             counter = et.SubElement(set, 'counter')
             counter.set('name', apply_aliases(mdapi_counter.get('ShortName'), aliases))
-            counter.set('symbol_name', mdapi_counter.get('SymbolName'))
+            counter.set('symbol_name', oa_registry.Registry.sanitize_symbol_name(mdapi_counter.get('SymbolName')))
             counter.set('underscore_name', underscore(mdapi_counter.get('SymbolName')))
             counter.set('description', apply_aliases(mdapi_counter.get('LongName'), aliases))
             counter.set('mdapi_group', apply_aliases(to_text(mdapi_counter.get('Group')), aliases))
@@ -848,10 +916,15 @@ for arg in args.xml:
             # delta equations are expected to be identical so warn if that's
             # not true...
             if bool(raw_read_eq) ^ bool(delta_read_eq) or raw_read_eq != delta_read_eq:
-                print_err("WARNING: Inconsistent raw and delta report equations for " + \
-                          mdapi_set.get('ShortName') + " :: " + mdapi_counter.get('SymbolName') + \
-                          "(" + mdapi_counter.get('ShortName') + ")" + ": raw=\"" + str(raw_read_eq) + \
-                          "\" delta=\"" + str(delta_read_eq) + "\" (SKIPPING)")
+                print_err(("WARNING: Inconsistent raw and delta report equations for {0} :: {1} ({2}): " +
+                           "raw=\"{3}\" / \"{4}\" delta=\"{5}\" / \"{6}\" (SKIPPING)")
+                          .format(mdapi_set.get('ShortName'),
+                                  mdapi_counter.get('SymbolName'),
+                                  mdapi_counter.get('ShortName'),
+                                  str(raw_read_eq),
+                                  mdapi_counter.get('SnapshotReportReadEquation'),
+                                  str(delta_read_eq),
+                                  mdapi_counter.get('DeltaReportReadEquation')))
                 set.remove(counter)
                 continue
 
@@ -901,7 +974,7 @@ for arg in args.xml:
                         #   print_err("WARNING: Counter equation (\"" + equation + "\") references un-kept raw equation of another counter : MetricSet=\"" + \
                         #             mdapi_set.get('ShortName') + "\" Metric=\"" + mdapi_counter.get('ShortName') + "\"")
 
-                    elif token[1:] not in raw_equations and token[1:] not in sys_vars:
+                    elif token[1:] not in raw_equations and not codegen.is_hw_var(token):
                         print_err("Unknown variable name: \"" + token + "\" in equation \"" + equation + "\"")
 
             symbol_name = counter.get('symbol_name')
@@ -910,7 +983,7 @@ for arg in args.xml:
             equation = equation.replace('$$', "$")
             for token in equation.split():
                 if token[0] == '$':
-                    if token[1:] not in counters and token[1:] not in sys_vars:
+                    if token[1:] not in counters and not codegen.is_hw_var(token):
                         print_err("WARNING: Counter equation (\"" + equation + "\") with unknown variable " + \
                                   token + " (maybe skipped counter): MetricSet=\"" + mdapi_set.get('ShortName') + \
                                   "\" Metric=\"" + mdapi_counter.get('SymbolName') + "(" + mdapi_counter.get('ShortName') + \
