@@ -1,6 +1,7 @@
 
 import copy
 import hashlib
+import re
 
 import xml.etree.ElementTree as et
 
@@ -71,13 +72,46 @@ class Registry:
 
         return hashlib.md5(registers_str).hexdigest()
 
-
     @staticmethod
-    def chipset_derive_hash(chipset, hash):
-        """Derive a HW config hash for a given chipset.
+    def chipset_derive_hash(chipset, set_name, hash):
+        """Derive a HW config hash for a given chipset & set name.
 
         This helps us avoiding collisions with identical config across
         different Gen or GT.
         """
 
-        return "%s-%s" % (chipset, hash)
+        return "%s-%s-%s" % (chipset, set_name, hash)
+
+
+    @staticmethod
+    def chipset_name(name):
+        known_chipsets = ( 'HSW',
+                           'BDW',
+                           'CHV',
+                           'SKL',
+                           'BXT',
+                           'KBL',
+                           'GLK',
+                           'CFL',
+                           'CNL',
+                           'ICL',
+                           'EHL',
+                           'TGL',
+                           'RKL',
+                           'DG1',
+                           'ACM',
+                           'PVC' )
+        if name in known_chipsets:
+            return name
+
+        # Unknown HW
+        assert 0
+
+
+    @staticmethod
+    def gt_name(name):
+        return re.sub(' ', '', name)
+
+    @staticmethod
+    def sanitize_symbol_name(text):
+        return text.replace('#', "_")
