@@ -28,6 +28,7 @@
 #include <pthread.h>
 #include <unistd.h>
 #include <signal.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -42,7 +43,11 @@
 
 #include "drm.h"
 #include "i915/gem.h"
+#include "i915/gem_create.h"
+#include "i915/gem_mman.h"
 #include "igt.h"
+#include "igt_aux.h"
+#include "igt_device_scan.h"
 
 #define OBJECT_SIZE (256 * 1024)
 
@@ -50,19 +55,22 @@
 #define BLT_WRITE_ALPHA		(1<<21)
 #define BLT_WRITE_RGB		(1<<20)
 
+IGT_TEST_DESCRIPTION("Test try to race gem_close against workload submission.");
+
 static uint32_t devid;
 static bool has_64bit_relocations;
-
-#define sigev_notify_thread_id _sigev_un._tid
+static bool has_softpin;
+static uint64_t exec_addr;
+static uint64_t data_addr;
 
 static void selfcopy(int fd, uint32_t ctx, uint32_t handle, int loops)
 {
 	struct drm_i915_gem_relocation_entry reloc[2];
 	struct drm_i915_gem_exec_object2 gem_exec[2];
 	struct drm_i915_gem_execbuffer2 execbuf;
-	struct drm_i915_gem_pwrite gem_pwrite;
 	struct drm_i915_gem_create create;
 	uint32_t buf[16], *b = buf;
+	int err;
 
 	memset(reloc, 0, sizeof(reloc));
 
@@ -78,9 +86,10 @@ static void selfcopy(int fd, uint32_t ctx, uint32_t handle, int loops)
 	reloc[0].target_handle = handle;
 	reloc[0].read_domains = I915_GEM_DOMAIN_RENDER;
 	reloc[0].write_domain = I915_GEM_DOMAIN_RENDER;
-	*b++ = 0;
+	reloc[0].presumed_offset = data_addr;
+	*b++ = data_addr;
 	if (has_64bit_relocations)
-		*b++ = 0;
+		*b++ = CANONICAL(data_addr) >> 32;
 
 	*b++ = 512 << 16;
 	*b++ = 4*1024;
@@ -89,9 +98,10 @@ static void selfcopy(int fd, uint32_t ctx, uint32_t handle, int loops)
 	reloc[1].target_handle = handle;
 	reloc[1].read_domains = I915_GEM_DOMAIN_RENDER;
 	reloc[1].write_domain = 0;
-	*b++ = 0;
+	reloc[1].presumed_offset = data_addr;
+	*b++ = data_addr;
 	if (has_64bit_relocations)
-		*b++ = 0;
+		*b++ = CANONICAL(data_addr) >> 32;
 
 	*b++ = MI_BATCH_BUFFER_END;
 	*b++ = 0;
@@ -104,10 +114,19 @@ static void selfcopy(int fd, uint32_t ctx, uint32_t handle, int loops)
 	create.size = 4096;
 	drmIoctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
 	gem_exec[1].handle = create.handle;
-	gem_exec[1].relocation_count = 2;
-	gem_exec[1].relocs_ptr = to_user_pointer(reloc);
+	gem_exec[1].offset = CANONICAL(exec_addr);
+	gem_exec[0].offset = CANONICAL(data_addr);
+	if (has_softpin) {
+		gem_exec[1].flags |= EXEC_OBJECT_PINNED | EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
+		gem_exec[0].flags |= EXEC_OBJECT_PINNED | EXEC_OBJECT_WRITE |
+				     EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
+	} else {
+		gem_exec[1].relocation_count = 2;
+		gem_exec[1].relocs_ptr = to_user_pointer(reloc);
+	}
 
 	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.flags |= I915_EXEC_NO_RELOC;
 	execbuf.buffers_ptr = to_user_pointer(gem_exec);
 	execbuf.buffer_count = 2;
 	execbuf.batch_len = (b - buf) * sizeof(*b);
@@ -115,15 +134,22 @@ static void selfcopy(int fd, uint32_t ctx, uint32_t handle, int loops)
 		execbuf.flags |= I915_EXEC_BLT;
 	execbuf.rsvd1 = ctx;
 
-	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
-	gem_pwrite.handle = create.handle;
-	gem_pwrite.offset = 0;
-	gem_pwrite.size = sizeof(buf);
-	gem_pwrite.data_ptr = to_user_pointer(buf);
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite) == 0) {
+	err = __gem_write(fd, create.handle, 0, buf, sizeof(buf));
+	if (err == -EOPNOTSUPP) {
+		void *ptr;
+
+		ptr = __gem_mmap__device_coherent(fd, create.handle, 0, sizeof(buf), PROT_WRITE);
+		if (!ptr) {
+			err = errno;
+		} else {
+			memcpy(ptr, buf, sizeof(buf));
+			gem_munmap(ptr, sizeof(buf));
+		}
+	}
+
+	if (!err)
 		while (loops-- && __gem_execbuf(fd, &execbuf) == 0)
 			;
-	}
 
 	drmIoctl(fd, DRM_IOCTL_GEM_CLOSE, &create.handle);
 }
@@ -229,6 +255,31 @@ static void thread(int fd, struct drm_gem_open name,
 	free(history);
 }
 
+static void multigpu_threads(int timeout, unsigned int flags, int gpu_count)
+{
+	int size = sysconf(_SC_NPROCESSORS_ONLN);
+
+	size /= gpu_count;
+	if (size < 1)
+		size = 1;
+
+	igt_multi_fork(gpu, gpu_count) {
+		struct drm_gem_open name;
+		int fd = __drm_open_driver_another(gpu, DRIVER_INTEL);
+
+		igt_assert(fd > 0);
+
+		igt_fork(child, size)
+			thread(fd, name, timeout, flags);
+
+		igt_waitchildren();
+		gem_quiescent_gpu(fd);
+		close(fd);
+	}
+
+	igt_waitchildren();
+}
+
 static void threads(int timeout, unsigned int flags)
 {
 	struct drm_gem_open name;
@@ -247,6 +298,8 @@ static void threads(int timeout, unsigned int flags)
 
 igt_main
 {
+	int gpu_count;
+
 	igt_fixture {
 		int fd;
 
@@ -255,11 +308,19 @@ igt_main
 
 		devid = intel_get_drm_devid(fd);
 		has_64bit_relocations = intel_gen(devid) >= 8;
+		has_softpin = !gem_has_relocations(fd);
+		exec_addr = gem_detect_safe_start_offset(fd);
+		data_addr = gem_detect_safe_alignment(fd);
+		exec_addr = max_t(exec_addr, exec_addr, data_addr);
+		data_addr += exec_addr;
+
+		gpu_count = igt_device_filter_count();
 
 		igt_fork_hang_detector(fd);
 		close(fd);
 	}
 
+	igt_describe("Basic workload submission.");
 	igt_subtest("basic-process") {
 		int fd = drm_open_driver(DRIVER_INTEL);
 
@@ -271,9 +332,35 @@ igt_main
 		close(fd);
 	}
 
+	igt_describe("Basic workload submission on multi-GPU machine.");
+	igt_subtest("multigpu-basic-process") {
+		igt_require(gpu_count > 1);
+
+		igt_multi_fork(child, gpu_count) {
+			int fd = __drm_open_driver_another(child, DRIVER_INTEL);
+
+			igt_assert(fd > 0);
+			process(fd, child);
+			gem_quiescent_gpu(fd);
+			close(fd);
+		}
+
+		igt_waitchildren();
+	}
+
+	igt_describe("Share buffer handle across different drm fd's and trying to race "
+		     " gem_close against continuous workload with minimum timeout.");
 	igt_subtest("basic-threads")
 		threads(1, 0);
 
+	igt_describe("Run basic-threads race on multi-GPU machine.");
+	igt_subtest("multigpu-basic-threads") {
+		igt_require(gpu_count > 1);
+		multigpu_threads(1, 0, gpu_count);
+	}
+
+	igt_describe("Test try to race gem_close against submission of continuous"
+		     " workload.");
 	igt_subtest("process-exit") {
 		int fd = drm_open_driver(DRIVER_INTEL);
 
@@ -285,9 +372,13 @@ igt_main
 		close(fd);
 	}
 
+	igt_describe("Share buffer handle across different drm fd's and trying to race"
+		     " gem_close against continuous workload in other contexts.");
 	igt_subtest("contexts")
 		threads(30, CONTEXTS);
 
+	igt_describe("Share buffer handle across different drm fd's and trying to race of"
+		     " gem_close against continuous workload.");
 	igt_subtest("gem-close-race")
 		threads(150, 0);
 
