@@ -25,25 +25,25 @@
  *
  */
 
-#include "igt.h"
-
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <time.h>
 
+#include "i915/gem.h"
+#include "i915/gem_create.h"
+#include "igt.h"
 #include "igt_rand.h"
 #include "sw_sync.h"
 
-#define LOCAL_I915_EXEC_BSD_SHIFT      (13)
-#define LOCAL_I915_EXEC_BSD_MASK       (3 << LOCAL_I915_EXEC_BSD_SHIFT)
+IGT_TEST_DESCRIPTION("Test the context create ioctls");
 
-#define ENGINE_FLAGS  (I915_EXEC_RING_MASK | LOCAL_I915_EXEC_BSD_MASK)
+#define ENGINE_FLAGS  (I915_EXEC_RING_MASK | I915_EXEC_BSD_MASK)
 
-static unsigned all_engines[16];
+static unsigned all_engines[I915_EXEC_RING_MASK + 1];
 static unsigned all_nengine;
 
-static unsigned ppgtt_engines[16];
+static unsigned ppgtt_engines[I915_EXEC_RING_MASK + 1];
 static unsigned ppgtt_nengine;
 
 static int create_ioctl(int fd, struct drm_i915_gem_context_create *arg)
@@ -81,7 +81,8 @@ static double elapsed(const struct timespec *start,
 	return (end->tv_sec - start->tv_sec) + 1e-9*(end->tv_nsec - start->tv_nsec);
 }
 
-static void files(int core, int timeout, const int ncpus)
+static void files(int core, const intel_ctx_cfg_t *cfg,
+		  int timeout, const int ncpus)
 {
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
@@ -100,18 +101,22 @@ static void files(int core, int timeout, const int ncpus)
 	igt_fork(child, ncpus) {
 		struct timespec start, end;
 		unsigned count = 0;
+		const intel_ctx_t *ctx;
 		int fd;
 
 		clock_gettime(CLOCK_MONOTONIC, &start);
 		do {
 			fd = gem_reopen_driver(core);
-			gem_context_copy_engines(core, 0, fd, 0);
+
+			ctx = intel_ctx_create(fd, cfg);
+			execbuf.rsvd1 = ctx->id;
 
 			obj.handle = gem_open(fd, name);
 			execbuf.flags &= ~ENGINE_FLAGS;
 			execbuf.flags |= ppgtt_engines[count % ppgtt_nengine];
 			gem_execbuf(fd, &execbuf);
 
+			intel_ctx_destroy(fd, ctx);
 			close(fd);
 			count++;
 
@@ -128,13 +133,14 @@ static void files(int core, int timeout, const int ncpus)
 	gem_close(core, batch);
 }
 
-static void active(int fd, const struct intel_execution_engine2 *e,
+static void active(int fd, const intel_ctx_cfg_t *cfg,
+		   const struct intel_execution_engine2 *e,
 		   int timeout, int ncpus)
 {
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj;
-	unsigned int nengine, engines[16];
+	unsigned int nengine, engines[ARRAY_SIZE(all_engines)];
 	unsigned *shared;
 	/* When e is NULL, test would run for all engines */
 	if (!e) {
@@ -160,19 +166,19 @@ static void active(int fd, const struct intel_execution_engine2 *e,
 	if (ncpus < 0) {
 		igt_fork(child, ppgtt_nengine) {
 			unsigned long count = 0;
-			int i915;
+			const intel_ctx_t *ctx;
 
-			i915 = gem_reopen_driver(fd);
 			/*
 			 * Ensure the gpu is idle by launching
 			 * a nop execbuf and stalling for it
 			 */
-			gem_quiescent_gpu(i915);
-			gem_context_copy_engines(fd, 0, i915, 0);
+			gem_quiescent_gpu(fd);
 
 			if (ppgtt_engines[child] == e->flags)
 				continue;
 
+			ctx = intel_ctx_create(fd, cfg);
+			execbuf.rsvd1 = ctx->id;
 			execbuf.flags = ppgtt_engines[child];
 
 			while (!READ_ONCE(*shared)) {
@@ -185,6 +191,7 @@ static void active(int fd, const struct intel_execution_engine2 *e,
 			}
 
 			igt_debug("hog[%d]: cycles=%lu\n", child, count);
+			intel_ctx_destroy(fd, ctx);
 		}
 		ncpus = -ncpus;
 	}
@@ -192,33 +199,27 @@ static void active(int fd, const struct intel_execution_engine2 *e,
 	igt_fork(child, ncpus) {
 		struct timespec start, end;
 		unsigned count = 0;
-		int i915;
-		uint32_t ctx;
 
-		i915 = gem_reopen_driver(fd);
 		/*
 		 * Ensure the gpu is idle by launching
 		 * a nop execbuf and stalling for it.
 		 */
-		gem_quiescent_gpu(i915);
-		ctx = gem_context_create(i915);
-		gem_context_copy_engines(fd, 0, i915, ctx);
+		gem_quiescent_gpu(fd);
 
 		clock_gettime(CLOCK_MONOTONIC, &start);
 		do {
-			execbuf.rsvd1 = gem_context_clone_with_engines(fd, ctx);
+			const intel_ctx_t *ctx = intel_ctx_create(fd, cfg);
+			execbuf.rsvd1 = ctx->id;
 			for (unsigned n = 0; n < nengine; n++) {
 				execbuf.flags = engines[n];
 				gem_execbuf(fd, &execbuf);
 			}
-			gem_context_destroy(fd, execbuf.rsvd1);
+			intel_ctx_destroy(fd, ctx);
 			count++;
 
 			clock_gettime(CLOCK_MONOTONIC, &end);
 		} while (elapsed(&start, &end) < timeout);
 
-		gem_context_destroy(fd, ctx);
-
 		gem_sync(fd, obj.handle);
 		clock_gettime(CLOCK_MONOTONIC, &end);
 		igt_info("[%d] Context creation + execution: %.3f us\n",
@@ -241,6 +242,15 @@ static void xchg_u32(void *array, unsigned i, unsigned j)
 	a[j] = tmp;
 }
 
+static void xchg_ptr(void *array, unsigned i, unsigned j)
+{
+	void **a = array, *tmp;
+
+	tmp = a[i];
+	a[i] = a[j];
+	a[j] = tmp;
+}
+
 static unsigned __context_size(int fd)
 {
 	switch (intel_gen(intel_get_drm_devid(fd))) {
@@ -273,22 +283,23 @@ static unsigned context_size(int fd)
 
 static uint64_t total_avail_mem(unsigned mode)
 {
-	uint64_t total = intel_get_avail_ram_mb();
+	uint64_t total = igt_get_avail_ram_mb();
 	if (mode & CHECK_SWAP)
-		total += intel_get_total_swap_mb();
+		total += igt_get_total_swap_mb();
 	return total << 20;
 }
 
-static void maximum(int fd, int ncpus, unsigned mode)
+static void maximum(int fd, const intel_ctx_cfg_t *cfg,
+		    int ncpus, unsigned mode)
 {
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj[2];
 	uint64_t avail_mem = total_avail_mem(mode);
 	unsigned ctx_size = context_size(fd);
-	uint32_t *contexts = NULL;
+	const intel_ctx_t **contexts = NULL;
 	unsigned long count = 0;
-	uint32_t ctx_id;
+	const intel_ctx_t *ctx;
 
 	do {
 		int err;
@@ -302,16 +313,14 @@ static void maximum(int fd, int ncpus, unsigned mode)
 
 		err = -ENOMEM;
 		if (avail_mem > (count + 1) * ctx_size)
-			err =  __gem_context_clone(fd, 0,
-						   I915_CONTEXT_CLONE_ENGINES,
-						   0, &ctx_id);
+			err = __intel_ctx_create(fd, cfg, &ctx);
 		if (err) {
 			igt_info("Created %lu contexts, before failing with '%s' [%d]\n",
 				 count, strerror(-err), -err);
 			break;
 		}
 
-		contexts[count++] = ctx_id;
+		contexts[count++] = ctx;
 	} while (1);
 	igt_require(count);
 
@@ -325,35 +334,26 @@ static void maximum(int fd, int ncpus, unsigned mode)
 
 	igt_fork(child, ncpus) {
 		struct timespec start, end;
-		int i915;
-
-		i915 = gem_reopen_driver(fd);
-		/*
-		 * Ensure the gpu is idle by launching
-		 * a nop execbuf and stalling for it.
-		 */
-		gem_quiescent_gpu(i915);
-		gem_context_copy_engines(fd, 0, i915, 0);
 
 		hars_petruska_f54_1_random_perturb(child);
-		obj[0].handle = gem_create(i915, 4096);
+		obj[0].handle = gem_create(fd, 4096);
 
 		clock_gettime(CLOCK_MONOTONIC, &start);
 		for (int repeat = 0; repeat < 3; repeat++) {
-			igt_permute_array(contexts, count, xchg_u32);
+			igt_permute_array(contexts, count, xchg_ptr);
 			igt_permute_array(all_engines, all_nengine, xchg_u32);
 
 			for (unsigned long i = 0; i < count; i++) {
-				execbuf.rsvd1 = contexts[i];
+				execbuf.rsvd1 = contexts[i]->id;
 				for (unsigned long j = 0; j < all_nengine; j++) {
 					execbuf.flags = all_engines[j];
-					gem_execbuf(i915, &execbuf);
+					gem_execbuf(fd, &execbuf);
 				}
 			}
 		}
-		gem_sync(i915, obj[0].handle);
+		gem_sync(fd, obj[0].handle);
 		clock_gettime(CLOCK_MONOTONIC, &end);
-		gem_close(i915, obj[0].handle);
+		gem_close(fd, obj[0].handle);
 
 		igt_info("[%d] Context execution: %.3f us\n", child,
 			 elapsed(&start, &end) / (3 * count * all_nengine) * 1e6);
@@ -363,7 +363,7 @@ static void maximum(int fd, int ncpus, unsigned mode)
 	gem_close(fd, obj[1].handle);
 
 	for (unsigned long i = 0; i < count; i++)
-		gem_context_destroy(fd, contexts[i]);
+		intel_ctx_destroy(fd, contexts[i]);
 	free(contexts);
 }
 
@@ -422,7 +422,7 @@ static void basic_ext_param(int i915)
 static void check_single_timeline(int i915, uint32_t ctx, int num_engines)
 {
 #define RCS_TIMESTAMP (0x2000 + 0x358)
-	const int gen = intel_gen(intel_get_drm_devid(i915));
+	const unsigned int gen = intel_gen(intel_get_drm_devid(i915));
 	const int has_64bit_reloc = gen >= 8;
 	struct drm_i915_gem_exec_object2 results = { .handle = gem_create(i915, 4096) };
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
@@ -563,6 +563,7 @@ igt_main
 	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
 	struct drm_i915_gem_context_create create;
 	const struct intel_execution_engine2 *e;
+	intel_ctx_cfg_t cfg;
 	int fd = -1;
 
 	igt_fixture {
@@ -570,7 +571,8 @@ igt_main
 		igt_require_gem(fd);
 		gem_require_contexts(fd);
 
-		__for_each_physical_engine(fd, e)
+		cfg = intel_ctx_cfg_all_physical(fd);
+		for_each_ctx_cfg_engine(fd, &cfg, e)
 			all_engines[all_nengine++] = e->flags;
 		igt_require(all_nengine);
 
@@ -585,6 +587,7 @@ igt_main
 		igt_fork_hang_detector(fd);
 	}
 
+	igt_describe("Test random context creation");
 	igt_subtest("basic") {
 		memset(&create, 0, sizeof(create));
 		create.ctx_id = rand();
@@ -594,45 +597,71 @@ igt_main
 		gem_context_destroy(fd, create.ctx_id);
 	}
 
+	igt_describe("Verify valid and invalid context extensions");
 	igt_subtest("ext-param")
 		basic_ext_param(fd);
+
+	igt_describe("Set, validate and execute particular context params");
 	igt_subtest("iris-pipeline")
 		iris_pipeline(fd);
 
+	igt_describe("Create contexts upto available RAM size, calculate the average "
+		     "performance of their execution on multiple parallel processes");
 	igt_subtest("maximum-mem")
-		maximum(fd, ncpus, CHECK_RAM);
-	igt_subtest("maximum-swap")
-		maximum(fd, ncpus, CHECK_RAM | CHECK_SWAP);
+		maximum(fd, &cfg, ncpus, CHECK_RAM);
 
+	igt_describe("Create contexts upto available RAM+SWAP size, calculate the average "
+		     "performance of their execution on multiple parallel processes");
+	igt_subtest("maximum-swap")
+		maximum(fd, &cfg, ncpus, CHECK_RAM | CHECK_SWAP);
+
+	igt_describe("Exercise implicit per-fd context creation");
 	igt_subtest("basic-files")
-		files(fd, 2, 1);
+		files(fd, &cfg, 2, 1);
+
+	igt_describe("Exercise implicit per-fd context creation on 1 CPU for long duration");
 	igt_subtest("files")
-		files(fd, 20, 1);
+		files(fd, &cfg, 20, 1);
+
+	igt_describe("Exercise implicit per-fd context creation on all CPUs for long duration");
 	igt_subtest("forked-files")
-		files(fd, 20, ncpus);
+		files(fd, &cfg, 20, ncpus);
 
 	/* NULL value means all engines */
+	igt_describe("Calculate the average performance of context creation and "
+		     "it's execution using all engines");
 	igt_subtest("active-all")
-		active(fd, NULL, 20, 1);
-	igt_subtest("forked-active-all")
-		active(fd, NULL, 20, ncpus);
+		active(fd, &cfg, NULL, 20, 1);
 
+	igt_describe("Calculate the average performance of context creation and it's execution "
+		     "using all engines on multiple parallel processes");
+	igt_subtest("forked-active-all")
+		active(fd, &cfg, NULL, 20, ncpus);
+
+	igt_describe("For each engine calculate the average performance of context creation "
+		     "execution and exercise context reclaim");
 	igt_subtest_with_dynamic("active") {
-		__for_each_physical_engine(fd, e) {
+		for_each_ctx_cfg_engine(fd, &cfg, e) {
 			igt_dynamic_f("%s", e->name)
-				active(fd, e, 20, 1);
+				active(fd, &cfg, e, 20, 1);
 		}
 	}
+
+	igt_describe("For each engine calculate the average performance of context creation "
+		     "and execution on multiple parallel processes");
 	igt_subtest_with_dynamic("forked-active") {
-		__for_each_physical_engine(fd, e) {
+		for_each_ctx_cfg_engine(fd, &cfg, e) {
 			igt_dynamic_f("%s", e->name)
-				active(fd, e, 20, ncpus);
+				active(fd, &cfg, e, 20, ncpus);
 		}
 	}
+
+	igt_describe("For each engine calculate the average performance of context creation "
+		     "and execution while all other engines are hogging the resources");
 	igt_subtest_with_dynamic("hog") {
-		__for_each_physical_engine(fd, e) {
+		for_each_ctx_cfg_engine(fd, &cfg, e) {
 			igt_dynamic_f("%s", e->name)
-				active(fd, e, 20, -1);
+				active(fd, &cfg, e, 20, -1);
 		}
 	}
 
