@@ -44,6 +44,7 @@
 
 #include "drm.h"
 #include "i915/gem.h"
+#include "i915/gem_create.h"
 #include "igt.h"
 
 #include "eviction_common.c"
@@ -143,8 +144,8 @@ static void test_forking_evictions(int fd, uint64_t size, uint64_t count,
 {
 	uint64_t trash_count;
 
-	trash_count = intel_get_total_ram_mb() * 11 / 10;
-	intel_require_memory(trash_count, size, CHECK_RAM | CHECK_SWAP);
+	trash_count = igt_get_total_ram_mb() * 11 / 10;
+	igt_require_memory(trash_count, size, CHECK_RAM | CHECK_SWAP);
 
 	forking_evictions(fd, &fault_ops, size, count, trash_count, flags);
 }
@@ -158,8 +159,8 @@ static void test_swapping_evictions(int fd, uint64_t size, uint64_t count)
 {
 	int trash_count;
 
-	trash_count = intel_get_total_ram_mb() * 11 / 10;
-	intel_require_memory(trash_count, size, CHECK_RAM | CHECK_SWAP);
+	trash_count = igt_get_total_ram_mb() * 11 / 10;
+	igt_require_memory(trash_count, size, CHECK_RAM | CHECK_SWAP);
 
 	swapping_evictions(fd, &fault_ops, size, count, trash_count);
 }
