@@ -42,6 +42,7 @@
 #include "drm.h"
 
 #include "i915/gem.h"
+#include "i915/gem_create.h"
 #include "igt.h"
 #include "igt_sysfs.h"
 #include "igt_x86.h"
@@ -53,6 +54,8 @@
 
 #define abs(x) ((x) >= 0 ? (x) : -(x))
 
+IGT_TEST_DESCRIPTION("Ensure that all operations around MMAP_GTT ioctl works.");
+
 static int OBJECT_SIZE = 16*1024*1024;
 
 static void
@@ -110,11 +113,11 @@ test_access(int fd)
 	mmap_arg.handle = handle;
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg);
 
-	igt_assert(mmap64(0, OBJECT_SIZE, PROT_READ | PROT_WRITE,
+	igt_assert(mmap(0, OBJECT_SIZE, PROT_READ | PROT_WRITE,
 			  MAP_SHARED, fd, mmap_arg.offset));
 
 	/* Check that the same offset on the other fd doesn't work. */
-	igt_assert(mmap64(0, OBJECT_SIZE, PROT_READ | PROT_WRITE,
+	igt_assert(mmap(0, OBJECT_SIZE, PROT_READ | PROT_WRITE,
 			  MAP_SHARED, fd2, mmap_arg.offset) == MAP_FAILED);
 	igt_assert(errno == EACCES);
 
@@ -125,7 +128,7 @@ test_access(int fd)
 
 	/* Recheck that it works after flink. */
 	/* Check that the same offset on the other fd doesn't work. */
-	igt_assert(mmap64(0, OBJECT_SIZE, PROT_READ | PROT_WRITE,
+	igt_assert(mmap(0, OBJECT_SIZE, PROT_READ | PROT_WRITE,
 			  MAP_SHARED, fd2, mmap_arg.offset));
 }
 
@@ -156,11 +159,11 @@ test_short(int fd)
 	for (pages = 1; pages <= OBJECT_SIZE / PAGE_SIZE; pages <<= 1) {
 		uint8_t *r, *w;
 
-		w = mmap64(0, pages * PAGE_SIZE, PROT_READ | PROT_WRITE,
+		w = mmap(0, pages * PAGE_SIZE, PROT_READ | PROT_WRITE,
 			   MAP_SHARED, fd, mmap_arg.offset);
 		igt_assert(w != MAP_FAILED);
 
-		r = mmap64(0, pages * PAGE_SIZE, PROT_READ,
+		r = mmap(0, pages * PAGE_SIZE, PROT_READ,
 			   MAP_SHARED, fd, mmap_arg.offset);
 		igt_assert(r != MAP_FAILED);
 
@@ -334,10 +337,12 @@ test_pf_nonblock(int i915)
 {
 	igt_spin_t *spin;
 	uint32_t *ptr;
+	uint64_t ahnd;
 
 	igt_require(mmap_gtt_version(i915) >= 3);
 
-	spin = igt_spin_new(i915);
+	ahnd = get_reloc_ahnd(i915, 0);
+	spin = igt_spin_new(i915, .ahnd = ahnd);
 
 	igt_set_timeout(1, "initial pagefaulting did not complete within 1s");
 
@@ -348,6 +353,7 @@ test_pf_nonblock(int i915)
 	igt_reset_timeout();
 
 	igt_spin_free(i915, spin);
+	put_ahnd(ahnd);
 }
 
 static void
@@ -378,13 +384,13 @@ test_isolation(int i915)
 
 	close(B);
 
-	ptr = mmap64(0, 4096, PROT_READ, MAP_SHARED, A, offset_a);
+	ptr = mmap(0, 4096, PROT_READ, MAP_SHARED, A, offset_a);
 	igt_assert(ptr != MAP_FAILED);
 	munmap(ptr, 4096);
 
 	close(A);
 
-	ptr = mmap64(0, 4096, PROT_READ, MAP_SHARED, A, offset_a);
+	ptr = mmap(0, 4096, PROT_READ, MAP_SHARED, A, offset_a);
 	igt_assert(ptr == MAP_FAILED);
 }
 
@@ -394,7 +400,7 @@ test_close_race(int i915)
 	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
 	_Atomic uint32_t *handles;
 
-	handles = mmap64(0, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	handles = mmap(0, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(handles != MAP_FAILED);
 
 	igt_fork(child, ncpus + 1) {
@@ -412,7 +418,7 @@ test_close_race(int i915)
 				  &mmap_arg) != -1) {
 				void *ptr;
 
-				ptr = mmap64(0, 4096,
+				ptr = mmap(0, 4096,
 					     PROT_WRITE, MAP_SHARED, i915,
 					     mmap_arg.offset);
 				if (ptr != MAP_FAILED) {
@@ -438,7 +444,7 @@ test_flink_race(int i915)
 	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
 	_Atomic uint32_t *handles;
 
-	handles = mmap64(0, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	handles = mmap(0, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(handles != MAP_FAILED);
 
 	igt_fork(child, ncpus + 1) {
@@ -463,7 +469,7 @@ test_flink_race(int i915)
 				  &mmap_arg) != -1) {
 				void *ptr;
 
-				ptr = mmap64(0, 4096,
+				ptr = mmap(0, 4096,
 					     PROT_WRITE, MAP_SHARED, fd,
 					     mmap_arg.offset);
 				if (ptr != MAP_FAILED) {
@@ -560,7 +566,7 @@ test_ptrace(int fd)
 	for (int i = 0; i < sz / sizeof(long); i++) {
 		long ret;
 
-		ret = ptrace(PTRACE_PEEKDATA, pid, gtt + i);
+		ret = ptrace(PTRACE_PEEKDATA, pid, gtt + i, (void *) 0);
 		igt_assert_eq_u64(ret, CC);
 		cpy[i] = ret;
 
@@ -736,14 +742,20 @@ static void
 test_hang_busy(int i915)
 {
 	uint32_t *ptr, *tile, *x;
+	const intel_ctx_t *ctx = intel_ctx_create(i915, NULL);
 	igt_spin_t *spin;
 	igt_hang_t hang;
 	uint32_t handle;
+	uint64_t ahnd;
 
-	hang = igt_allow_hang(i915, 0, 0);
+	hang = igt_allow_hang(i915, ctx->id, 0);
 	igt_require(igt_params_set(i915, "reset", "1")); /* global */
 
-	spin = igt_spin_new(i915, .flags = IGT_SPIN_POLL_RUN | IGT_SPIN_FENCE_OUT | IGT_SPIN_NO_PREEMPTION);
+	ahnd = get_reloc_ahnd(i915, ctx->id);
+	spin = igt_spin_new(i915, .ctx = ctx, .ahnd = ahnd,
+			    .flags = IGT_SPIN_POLL_RUN |
+				     IGT_SPIN_FENCE_OUT |
+				     IGT_SPIN_NO_PREEMPTION);
 	igt_spin_busywait_until_started(spin);
 	igt_assert(spin->execbuf.buffer_count == 2);
 
@@ -783,21 +795,29 @@ test_hang_busy(int i915)
 	munmap(ptr, 4096);
 
 	igt_spin_free(i915, spin);
+	put_ahnd(ahnd);
 	igt_disallow_hang(i915, hang);
+	intel_ctx_destroy(i915, ctx);
 }
 
 static void
 test_hang_user(int i915)
 {
+	const intel_ctx_t *ctx = intel_ctx_create(i915, NULL);
 	uint32_t *ptr, *mem, *x;
 	igt_spin_t *spin;
 	igt_hang_t hang;
 	uint32_t handle;
+	uint64_t ahnd;
 
-	hang = igt_allow_hang(i915, 0, 0);
+	hang = igt_allow_hang(i915, ctx->id, 0);
 	igt_require(igt_params_set(i915, "reset", "1")); /* global */
 
-	spin = igt_spin_new(i915, .flags = IGT_SPIN_POLL_RUN | IGT_SPIN_FENCE_OUT | IGT_SPIN_NO_PREEMPTION);
+	ahnd = get_reloc_ahnd(i915, ctx->id);
+	spin = igt_spin_new(i915, .ctx = ctx, .ahnd = ahnd,
+			    .flags = IGT_SPIN_POLL_RUN |
+				     IGT_SPIN_FENCE_OUT |
+				     IGT_SPIN_NO_PREEMPTION);
 	igt_spin_busywait_until_started(spin);
 	igt_assert(spin->execbuf.buffer_count == 2);
 
@@ -833,7 +853,9 @@ test_hang_user(int i915)
 	munmap(ptr, 4096);
 
 	igt_spin_free(i915, spin);
+	put_ahnd(ahnd);
 	igt_disallow_hang(i915, hang);
+	intel_ctx_destroy(i915, ctx);
 }
 
 static int min_tile_width(uint32_t devid, int tiling)
@@ -926,7 +948,7 @@ test_huge_bo(int fd, int huge, int tiling)
 		size = gem_global_aperture_size(fd) + PAGE_SIZE;
 		break;
 	}
-	intel_require_memory(1, size, CHECK_RAM);
+	igt_require_memory(1, size, CHECK_RAM);
 
 	last_offset = size - PAGE_SIZE;
 
@@ -1013,11 +1035,11 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
 		huge_object_size = gem_global_aperture_size(fd) + PAGE_SIZE;
 		break;
 	default:
-		huge_object_size = (intel_get_total_ram_mb() << 19) + PAGE_SIZE;
+		huge_object_size = (igt_get_total_ram_mb() << 19) + PAGE_SIZE;
 		mode |= CHECK_SWAP;
 		break;
 	}
-	intel_require_memory(2*ncpus, huge_object_size, mode);
+	igt_require_memory(2*ncpus, huge_object_size, mode);
 
 	igt_fork(child, ncpus) {
 		uint64_t valid_size = huge_object_size;
@@ -1252,6 +1274,7 @@ igt_main
 		gem_require_mappable_ggtt(fd);
 	}
 
+	igt_describe("Verify mapping to invalid gem objects fails.");
 	igt_subtest("bad-object") {
 		uint32_t real_handle = gem_create(fd, 4096);
 		uint32_t handles[20];
@@ -1273,77 +1296,126 @@ igt_main
 		gem_close(fd, real_handle);
 	}
 
+	igt_describe("Basic checks of GEM_MMAP_GTT ioctl.");
 	igt_subtest("basic")
 		test_access(fd);
+	igt_describe("Test mmaping less than the full object.");
 	igt_subtest("basic-short")
 		test_short(fd);
+	igt_describe("Test copy between two GTT mmappings.");
 	igt_subtest("basic-copy")
 		test_copy(fd);
+	igt_describe("Test to read content from GTT mmapped object.");
 	igt_subtest("basic-read")
 		test_read(fd);
+	igt_describe("Test to write content to GTT mmapped object.");
 	igt_subtest("basic-write")
 		test_write(fd);
+	igt_describe("Test creates a prefault object into GTT and "
+		     "writes into it from another GTT mmapped.");
 	igt_subtest("basic-write-gtt")
 		test_write_gtt(fd);
+	igt_describe("Inspect a GTT mmap using ptrace().");
 	igt_subtest("ptrace")
 		test_ptrace(fd);
+	igt_describe("Check whether a write through the GTT is immediately visible to the CPU.");
 	igt_subtest("coherency")
 		test_coherency(fd);
+	igt_describe("Check the userspace clflushing of the GTT mmap.");
 	igt_subtest("clflush")
 		test_clflush(fd);
+	igt_describe("Check read/writes across a GPU reset.");
 	igt_subtest("hang")
 		test_hang(fd);
+	igt_describe("Exercise the GTT mmap revocation for a reset on a busy object.");
 	igt_subtest("hang-busy")
 		test_hang_busy(fd);
+	igt_describe("Mix a busy hang with GTT and userptr.");
 	igt_subtest("hang-user")
 		test_hang_user(fd);
+	igt_describe("Check basic read->write order of a GTT mmapped bo.");
 	igt_subtest("basic-read-write")
 		test_read_write(fd, READ_BEFORE_WRITE);
+	igt_describe("Check basic write->read order of a GTT mmapped bo.");
 	igt_subtest("basic-write-read")
 		test_read_write(fd, READ_AFTER_WRITE);
+	igt_describe("Check distinct read->write order of a GTT mmapped bo.");
 	igt_subtest("basic-read-write-distinct")
 		test_read_write2(fd, READ_BEFORE_WRITE);
+	igt_describe("Check distinct write->read order of a GTT mmapped bo.");
 	igt_subtest("basic-write-read-distinct")
 		test_read_write2(fd, READ_AFTER_WRITE);
+	igt_describe("Excercise concurrent pagefaulting of a GTT mmaped bo.");
 	igt_subtest("fault-concurrent")
 		test_fault_concurrent(fd, I915_TILING_NONE);
+	igt_describe("Excercise concurrent pagefaulting of a X-tiled GTT mmaped bo.");
 	igt_subtest("fault-concurrent-X")
 		test_fault_concurrent(fd, I915_TILING_X);
+	igt_describe("Excercise concurrent pagefaulting of a Y-tiled GTT mmaped bo.");
 	igt_subtest("fault-concurrent-Y")
 		test_fault_concurrent(fd, I915_TILING_Y);
+	igt_describe("Check coherency between GTT and CPU mmappings with LLC.");
 	igt_subtest("basic-write-cpu-read-gtt")
 		test_write_cpu_read_gtt(fd);
+	igt_describe("Check the performance of WC writes with WC reads of GTT "
+		     "and WC writes of GTT with WB writes of CPU.");
 	igt_subtest("basic-wc")
 		test_wc(fd);
+	igt_describe("Test mmap_offset lifetime, closing the object on"
+		     " another file should not affect the local mmap_offset.");
 	igt_subtest("isolation")
 		test_isolation(fd);
+	igt_describe("Test MMAP_GTT extension validity.");
 	igt_subtest("zero-extend")
 		test_zero_extend(fd);
+	igt_describe("Test to check that a few threads opening and closing handles"
+		     " cause explosion in other threads in the process of mmaping that handle.");
 	igt_subtest("close-race")
 		test_close_race(fd);
+	igt_describe("Test to check that a few threads opening and closing flink handles"
+		     " cause explosion in other threads in the process of mmaping that handle.");
 	igt_subtest("flink-race")
 		test_flink_race(fd);
+	igt_describe("Check that the initial pagefault is non-blocking.");
 	igt_subtest("pf-nonblock")
 		test_pf_nonblock(fd);
 
+	igt_describe("Check mmap access to a small buffer object by CPU directly"
+		     " and through GTT in sequence.");
 	igt_subtest("basic-small-bo")
 		test_huge_bo(fd, -1, I915_TILING_NONE);
+	igt_describe("Check mmap access to a small X-tiled buffer object by CPU "
+		     "directly and through GTT in sequence.");
 	igt_subtest("basic-small-bo-tiledX")
 		test_huge_bo(fd, -1, I915_TILING_X);
+	igt_describe("Check mmap access to a small Y-tiled buffer object by CPU "
+		     "directly and through GTT in sequence.");
 	igt_subtest("basic-small-bo-tiledY")
 		test_huge_bo(fd, -1, I915_TILING_Y);
 
+	igt_describe("Check mmap access to a big buffer object by CPU directly "
+		     "and through GTT in sequence.");
 	igt_subtest("big-bo")
 		test_huge_bo(fd, 0, I915_TILING_NONE);
+	igt_describe("Check mmap access to a big X-tiled buffer object by CPU "
+		     "directly and through GTT in sequence.");
 	igt_subtest("big-bo-tiledX")
 		test_huge_bo(fd, 0, I915_TILING_X);
+	igt_describe("Check mmap access to a big Y-tiled buffer object by CPU "
+		     "directly and through GTT in sequence.");
 	igt_subtest("big-bo-tiledY")
 		test_huge_bo(fd, 0, I915_TILING_Y);
 
+	igt_describe("Check mmap access to a huge buffer object by CPU directly "
+		     "and through GTT in sequence.");
 	igt_subtest("huge-bo")
 		test_huge_bo(fd, 1, I915_TILING_NONE);
+	igt_describe("Check mmap access to a huge X-tiled buffer object by CPU "
+		     "directly and through GTT in sequence.");
 	igt_subtest("huge-bo-tiledX")
 		test_huge_bo(fd, 1, I915_TILING_X);
+	igt_describe("Check mmap access to a huge Y-tiled buffer object by CPU "
+		     "directly and through GTT in sequence.");
 	igt_subtest("huge-bo-tiledY")
 		test_huge_bo(fd, 1, I915_TILING_Y);
 
@@ -1351,27 +1423,33 @@ igt_main
 		const struct copy_size {
 			const char *prefix;
 			int size;
+			const char *description;
 		} copy_sizes[] = {
-			{ "basic-small", -2 },
-			{ "medium", -1 },
-			{ "big", 0 },
-			{ "huge", 1 },
-			{ "swap", 2 },
+			{ "basic-small", -2, "small bo's." },
+			{ "medium", -1, "medium bo's." },
+			{ "big", 0, "big bo's." },
+			{ "huge", 1, "huge bo's." },
+			{ "swap", 2, "huge bo's larger than physical memory"
+				     " and resulting in thrashing of swap space." },
 			{ }
 		};
 		const struct copy_mode {
 			const char *suffix;
 			int tiling_x, tiling_y;
+			const char *description;
 		} copy_modes[] = {
-			{ "", I915_TILING_NONE, I915_TILING_NONE},
-			{ "-XY", I915_TILING_X, I915_TILING_Y},
-			{ "-odd", -I915_TILING_X, -I915_TILING_Y},
+			{ "", I915_TILING_NONE, I915_TILING_NONE, "normal"},
+			{ "-XY", I915_TILING_X, I915_TILING_Y, "tiled"},
+			{ "-odd", -I915_TILING_X, -I915_TILING_Y, "odd tiled"},
 			{}
 		};
 		const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
 
 		for (const struct copy_size *s = copy_sizes; s->prefix; s++)
 			for (const struct copy_mode *m = copy_modes; m->suffix; m++) {
+				igt_describe_f("Check page by page copying between two GTT"
+					       " mmapped %s-%s", m->description,
+					       s->description);
 				igt_subtest_f("%s-copy%s", s->prefix, m->suffix)
 					test_huge_copy(fd,
 						       s->size,
@@ -1379,6 +1457,10 @@ igt_main
 						       m->tiling_y,
 						       1);
 
+				igt_describe_f("Add forked contention with lighter variant"
+					       " (single cpu) and check page by page copying"
+					       " between two GTT mmapped %s-%s",
+					       m->description, s->description);
 				igt_subtest_f("cpuset-%s-copy%s", s->prefix, m->suffix) {
 					cpu_set_t cpu, old;
 
@@ -1396,6 +1478,9 @@ igt_main
 					igt_assert(sched_setaffinity(0, sizeof(old), &old) == 0);
 				}
 
+				igt_describe_f("Add forked contention and check page by page"
+					       " copying between two GTT mmapped %s-%s",
+					       m->description, s->description);
 				igt_subtest_f("forked-%s-copy%s", s->prefix, m->suffix)
 					test_huge_copy(fd,
 						       s->size,
