@@ -25,6 +25,8 @@
  *
  */
 
+#include <linux/userfaultfd.h>
+
 #include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
@@ -34,10 +36,15 @@
 #include <fcntl.h>
 #include <inttypes.h>
 #include <errno.h>
-#include <sys/stat.h>
 #include <sys/ioctl.h>
+#include <sys/resource.h>
+#include <sys/stat.h>
 #include <sys/time.h>
+#include <pthread.h>
+
 #include "drm.h"
+#include "i915/gem_create.h"
+#include "igt_vgem.h"
 
 #define MiB(x) ((x) * 1024 * 1024)
 
@@ -72,8 +79,151 @@ static void pread_self(int i915)
 	}
 }
 
+static int userfaultfd(int flags)
+{
+	return syscall(SYS_userfaultfd, flags);
+}
+
+struct ufd_thread {
+	uint32_t *page;
+	int i915;
+	int vgem;
+	int err;
+};
+
+static uint32_t dmabuf_create_handle(int i915, int vgem)
+{
+	struct vgem_bo scratch;
+	uint32_t handle;
+	int dmabuf;
+
+	scratch.width = 64;
+	scratch.height = 64;
+	scratch.bpp = 32;
+	vgem_create(vgem, &scratch);
+
+	dmabuf = prime_handle_to_fd(vgem, scratch.handle);
+	handle = prime_fd_to_handle(i915, dmabuf);
+	close(dmabuf);
+
+	return handle;
+}
+
+static void *ufd_thread(void *arg)
+{
+	struct ufd_thread *t = arg;
+	uint32_t handle = dmabuf_create_handle(t->i915, t->vgem);
+
+	t->err = __gem_read(t->i915, handle, 0, t->page, 1);
+	gem_close(t->i915, handle);
+
+	return NULL;
+}
+
+static void write_value(const char *path, int value)
+{
+	char buf[80];
+	int fd, len;
+
+	len = sprintf(buf, "%d", value);
+	if (len < 0)
+		return;
+
+	fd = open(path, O_WRONLY);
+	if (fd != -1) {
+		write(fd, buf, len);
+		close(fd);
+	}
+}
+
+static void unlimited_processes(unsigned int limit)
+{
+	struct rlimit rlim;
+
+	write_value("/proc/sys/kernel/threads-max", 150000);
+	write_value("/proc/sys/vm/max_map_count", 500000);
+	write_value("/proc/sys/kernel/pid_max", 200000);
+
+	if (getrlimit(RLIMIT_NPROC, &rlim))
+		return;
+
+	rlim.rlim_cur = limit;
+	rlim.rlim_max = limit;
+	setrlimit(RLIMIT_NPROC, &rlim);
+}
+
+static void test_exhaustion(int i915)
+{
+	struct uffdio_api api = { .api = UFFD_API };
+	struct uffdio_register reg;
+	struct uffdio_copy copy;
+	struct ufd_thread t = {
+		.i915 = i915,
+		.vgem = drm_open_driver(DRIVER_VGEM),
+	};
+	pthread_t *thread = NULL;
+	struct uffd_msg msg;
+	unsigned long count;
+	char buf[4096];
+	int ufd;
+
+	unlimited_processes(1024 * 1024);
+
+	ufd = userfaultfd(0);
+	igt_require_f(ufd != -1, "kernel support for userfaultfd\n");
+	igt_require_f(ioctl(ufd, UFFDIO_API, &api) == 0 && api.api == UFFD_API,
+		      "userfaultfd API v%lld:%lld\n", UFFD_API, api.api);
+
+	t.page = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, 0, 0);
+	igt_assert(t.page != MAP_FAILED);
+
+	/* Register our fault handler for t.page */
+	memset(&reg, 0, sizeof(reg));
+	reg.mode = UFFDIO_REGISTER_MODE_MISSING;
+	reg.range.start = to_user_pointer(t.page);
+	reg.range.len = 4096;
+	do_ioctl(ufd, UFFDIO_REGISTER, &reg);
+
+	count = 0;
+	while (!READ_ONCE(t.err)) {
+		if (is_power_of_two(count)) {
+			unsigned long sz = count ? 2 * count : 1;
+			thread = realloc(thread, sz * sizeof(*thread));
+			igt_assert(thread);
+		}
+		if (pthread_create(&thread[count], NULL, ufd_thread, &t))
+			break;
+
+		if (count == 0) { /* Wait for the first userfault */
+			igt_assert_eq(read(ufd, &msg, sizeof(msg)), sizeof(msg));
+			igt_assert_eq(msg.event, UFFD_EVENT_PAGEFAULT);
+			igt_assert(from_user_pointer(msg.arg.pagefault.address) == t.page);
+		}
+
+		count++;
+	}
+	igt_assert(count);
+	if (t.err)
+		igt_warn("err:%d after %lu threads\n", t.err, count);
+
+	/* Service the fault; releasing the stuck ioctls */
+	memset(&copy, 0, sizeof(copy));
+	copy.dst = msg.arg.pagefault.address;
+	copy.src = to_user_pointer(memset(buf, 0xc5, sizeof(buf)));
+	copy.len = 4096;
+	do_ioctl(ufd, UFFDIO_COPY, &copy);
+
+	while (count--)
+		pthread_join(thread[count], NULL);
+	free(thread);
+
+	munmap(t.page, 4096);
+	close(ufd);
+
+	close(t.vgem);
+}
+
 #define OBJECT_SIZE 16384
-#define LARGE_OBJECT_SIZE 1024 * 1024
 #define KGRN "\x1B[32m"
 #define KRED "\x1B[31m"
 #define KNRM "\x1B[0m"
@@ -110,10 +260,7 @@ static const char *bytes_per_sec(char *buf, double v)
 	return buf;
 }
 
-
 uint32_t *src, dst;
-uint32_t *dst_user, src_stolen, large_stolen;
-uint32_t *stolen_pf_user, *stolen_nopf_user;
 int fd, count;
 int object_size = 0;
 
@@ -153,11 +300,10 @@ igt_main_args("s:", NULL, help_str, opt_handler, NULL)
 
 	igt_fixture {
 		fd = drm_open_driver(DRIVER_INTEL);
+		gem_require_pread_pwrite(fd);
 
 		dst = gem_create(fd, object_size);
 		src = malloc(object_size);
-		src_stolen = gem_create_stolen(fd, object_size);
-		dst_user = malloc(object_size);
 	}
 
 	igt_subtest("bench") {
@@ -178,6 +324,9 @@ igt_main_args("s:", NULL, help_str, opt_handler, NULL)
 	igt_subtest("self")
 		pread_self(fd);
 
+	igt_subtest("exhaustion")
+		test_exhaustion(fd);
+
 	for (c = cache; c->level != -1; c++) {
 		igt_subtest(c->name) {
 			gem_set_caching(fd, dst, c->level);
@@ -197,96 +346,9 @@ igt_main_args("s:", NULL, help_str, opt_handler, NULL)
 		}
 	}
 
-	igt_subtest("stolen-normal") {
-		gem_require_stolen_support(fd);
-		for (count = 1; count <= 1<<17; count <<= 1) {
-			struct timeval start, end;
-
-			gettimeofday(&start, NULL);
-			do_gem_read(fd, src_stolen, dst_user, object_size, count);
-			gettimeofday(&end, NULL);
-			usecs = elapsed(&start, &end, count);
-			bps = bytes_per_sec(buf, object_size/usecs*1e6);
-			igt_info("Time to pread %d bytes x %6d:	%7.3fµs, %s\n",
-				 object_size, count, usecs, bps);
-			fflush(stdout);
-		}
-	}
-	for (c = cache; c->level != -1; c++) {
-		igt_subtest_f("stolen-%s", c->name) {
-			gem_require_stolen_support(fd);
-			gem_set_caching(fd, src_stolen, c->level);
-
-			for (count = 1; count <= 1<<17; count <<= 1) {
-				struct timeval start, end;
-
-				gettimeofday(&start, NULL);
-				do_gem_read(fd, src_stolen, dst_user,
-					    object_size, count);
-				gettimeofday(&end, NULL);
-				usecs = elapsed(&start, &end, count);
-				bps = bytes_per_sec(buf, object_size/usecs*1e6);
-				igt_info("Time to stolen-%s pread %d bytes x %6d:      %7.3fµs, %s\n",
-					 c->name, object_size, count, usecs, bps);
-				fflush(stdout);
-			}
-		}
-	}
-
-	/* List the time taken in pread operation for stolen objects, with
-	 * and without the overhead of page fault handling on accessing the
-	 * user space buffer
-	 */
-	igt_subtest("pagefault-pread") {
-		gem_require_stolen_support(fd);
-		large_stolen = gem_create_stolen(fd, LARGE_OBJECT_SIZE);
-		stolen_nopf_user = (uint32_t *) mmap(NULL, LARGE_OBJECT_SIZE,
-						PROT_WRITE,
-						MAP_ANONYMOUS|MAP_PRIVATE,
-						-1, 0);
-		igt_assert(stolen_nopf_user);
-
-		for (count = 1; count <= 10; count ++) {
-			struct timeval start, end;
-			double t_elapsed = 0;
-
-			gettimeofday(&start, NULL);
-			do_gem_read(fd, large_stolen, stolen_nopf_user,
-				    LARGE_OBJECT_SIZE, 1);
-			gettimeofday(&end, NULL);
-			t_elapsed = elapsed(&start, &end, count);
-			bps = bytes_per_sec(buf, object_size/t_elapsed*1e6);
-			igt_info("Pagefault-N - Time to pread %d bytes: %7.3fµs, %s\n",
-				 LARGE_OBJECT_SIZE, t_elapsed, bps);
-
-			stolen_pf_user = (uint32_t *) mmap(NULL, LARGE_OBJECT_SIZE,
-						      PROT_WRITE,
-						      MAP_ANONYMOUS|MAP_PRIVATE,
-						      -1, 0);
-			igt_assert(stolen_pf_user);
-
-			gettimeofday(&start, NULL);
-			do_gem_read(fd, large_stolen, stolen_pf_user,
-				    LARGE_OBJECT_SIZE, 1);
-			gettimeofday(&end, NULL);
-			usecs = elapsed(&start, &end, count);
-			bps = bytes_per_sec(buf, object_size/usecs*1e6);
-			igt_info("Pagefault-Y - Time to pread %d bytes: %7.3fµs, %s%s%s\n",
-				 LARGE_OBJECT_SIZE, usecs,
-				 t_elapsed < usecs ? KGRN : KRED, bps, KNRM);
-			fflush(stdout);
-			munmap(stolen_pf_user, LARGE_OBJECT_SIZE);
-		}
-		munmap(stolen_nopf_user, LARGE_OBJECT_SIZE);
-		gem_close(fd, large_stolen);
-	}
-
-
 	igt_fixture {
 		free(src);
 		gem_close(fd, dst);
-		free(dst_user);
-		gem_close(fd, src_stolen);
 
 		close(fd);
 	}
