@@ -44,6 +44,7 @@
 
 #include "drm.h"
 #include "i915/gem.h"
+#include "i915/gem_create.h"
 #include "i915_reg.h"
 #include "igt.h"
 
@@ -327,7 +328,7 @@ igt_simple_main
 	igt_require_gem(fd);
 
 	count = 3 + gem_aperture_size(fd) / (1024 * 1024);
-	intel_require_memory(count, 1024*1024, CHECK_RAM);
+	igt_require_memory(count, 1024*1024, CHECK_RAM);
 
 	handle = malloc(sizeof(uint32_t)*count*2);
 	start_val = handle + count;
