@@ -40,6 +40,7 @@
 
 #include "igt_rand.h"
 #include "igt_device.h"
+#include "i915/intel_memory_region.h"
 
 uint32_t gem_bo;
 uint32_t gem_bo_small;
@@ -56,21 +57,9 @@ static int legacy_addfb(int fd, struct drm_mode_fb_cmd *arg)
 	return err;
 }
 
-static int rmfb(int fd, uint32_t id)
-{
-	int err;
-
-	err = 0;
-	if (igt_ioctl(fd, DRM_IOCTL_MODE_RMFB, &id))
-		err = -errno;
-
-	errno = 0;
-	return err;
-}
-
 static void invalid_tests(int fd)
 {
-	struct local_drm_mode_fb_cmd2 f = {};
+	struct drm_mode_fb_cmd2 f = {};
 
 	f.width = 512;
 	f.height = 512;
@@ -87,60 +76,78 @@ static void invalid_tests(int fd)
 
 		f.handles[0] = gem_bo;
 
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 		f.fb_id = 0;
 	}
 
-	f.flags = LOCAL_DRM_MODE_FB_MODIFIERS;
+	f.flags = DRM_MODE_FB_MODIFIERS;
 
+	igt_describe("Test that addfb2 call fails correctly for unused handle");
 	igt_subtest("unused-handle") {
 		igt_require_fb_modifiers(fd);
 
 		f.handles[1] = gem_bo_small;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		f.handles[1] = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for unused pitches");
 	igt_subtest("unused-pitches") {
 		igt_require_fb_modifiers(fd);
 
 		f.pitches[1] = 512;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		f.pitches[1] = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for unused offset");
 	igt_subtest("unused-offsets") {
 		igt_require_fb_modifiers(fd);
 
 		f.offsets[1] = 512;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		f.offsets[1] = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for unused modifier");
 	igt_subtest("unused-modifier") {
 		igt_require_fb_modifiers(fd);
 
-		f.modifier[1] =  LOCAL_I915_FORMAT_MOD_X_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		f.modifier[1] =  I915_FORMAT_MOD_X_TILED;
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		f.modifier[1] = 0;
 	}
 
+	igt_describe("Check if addfb2 call works for clobbered modifier");
 	igt_subtest("clobberred-modifier") {
 		igt_require_intel(fd);
+		igt_require(gem_available_fences(fd) > 0);
 		f.flags = 0;
 		f.modifier[0] = 0;
 		gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4);
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 		f.fb_id = 0;
-		igt_assert(f.modifier[0] == 0);
+		igt_assert_eq(f.modifier[0], 0);
 	}
 
+	igt_describe("Check if addfb2 with a system memory gem object "
+		     "fails correctly if device requires local memory framebuffers");
+	igt_subtest("invalid-smem-bo-on-discrete") {
+		uint32_t handle, stride;
+		uint64_t size;
+
+		igt_require_intel(fd);
+		igt_require(gem_has_lmem(fd));
+		igt_calc_fb_size(fd, f.width, f.height,
+				DRM_FORMAT_XRGB8888, 0, &size, &stride);
+		handle = gem_create_in_memory_regions(fd, size, REGION_SMEM);
+		f.handles[0] = handle;
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EREMOTE);
+	}
+
+	igt_describe("Check if addfb2 call works for legacy formats");
 	igt_subtest("legacy-format") {
 		struct {
 			/* drm_mode_legacy_fb_format() */
@@ -176,7 +183,7 @@ static void invalid_tests(int fd)
 			igt_debug("{bpp:%d, depth:%d} -> expect:%d\n",
 				  arg.bpp, arg.depth, known_formats[i].expect);
 			if (arg.fb_id) {
-				igt_assert_eq(rmfb(fd, arg.fb_id), 0);
+				do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &arg.fb_id);
 				arg.fb_id = 0;
 			}
 		}
@@ -213,11 +220,18 @@ static void invalid_tests(int fd)
 				     "Expected %d with {bpp:%d, depth:%d}, got %d instead\n",
 				     expect, arg.bpp, arg.depth, err);
 			if (arg.fb_id) {
-				igt_assert_eq(rmfb(fd, arg.fb_id), 0);
+				do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &arg.fb_id);
 				arg.fb_id = 0;
 			}
 
 			count++;
+
+			/*
+			 * to avoid exessive logging stop after 10k loops
+			 */
+			if (count >= 10000) {
+				break;
+			}
 		}
 
 		/* After all the abuse, confirm the known_formats */
@@ -234,7 +248,7 @@ static void invalid_tests(int fd)
 				     arg.bpp, arg.depth,
 				     err);
 			if (arg.fb_id) {
-				igt_assert_eq(rmfb(fd, arg.fb_id), 0);
+				do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &arg.fb_id);
 				arg.fb_id = 0;
 			}
 		}
@@ -266,23 +280,29 @@ static void pitch_tests(int fd)
 		igt_assert(gem_bo);
 	}
 
+	igt_describe("Test that addfb2 call fails correctly without handle");
 	igt_subtest("no-handle") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 	}
 
+	igt_describe("Check if addfb2 call works with given handle");
 	f.handles[0] = gem_bo;
 	igt_subtest("basic") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 		f.fb_id = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for bad-pitches");
 	for (i = 0; i < ARRAY_SIZE(bad_pitches); i++) {
 		igt_subtest_f("bad-pitch-%i", bad_pitches[i]) {
 			f.pitches[0] = bad_pitches[i];
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-				   errno == EINVAL);
+			igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f), -1);
+			if (is_nouveau_device(fd) && bad_pitches[i] > 4 * 1024)
+				igt_assert_eq(errno, ERANGE);
+			else
+				igt_assert_eq(errno, EINVAL);
+			errno = 0;
 		}
 	}
 
@@ -305,12 +325,12 @@ static void tiling_tests(int fd)
 		igt_fixture {
 			igt_require_intel(fd);
 			tiled_x_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
-				DRM_FORMAT_XRGB8888, LOCAL_I915_FORMAT_MOD_X_TILED,
+				DRM_FORMAT_XRGB8888, I915_FORMAT_MOD_X_TILED,
 				1024*4, NULL, NULL, NULL);
 			igt_assert(tiled_x_bo);
 
 			tiled_y_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
-				DRM_FORMAT_XRGB8888, LOCAL_I915_FORMAT_MOD_Y_TILED,
+				DRM_FORMAT_XRGB8888, I915_FORMAT_MOD_Y_TILED,
 				1024*4, NULL, NULL, NULL);
 			igt_assert(tiled_y_bo);
 
@@ -320,39 +340,46 @@ static void tiling_tests(int fd)
 		}
 
 		f.pitches[0] = 1024*4;
-		igt_subtest("basic-X-tiled") {
+		igt_describe("Check if addfb2 and rmfb call works for basic x-tiling test");
+		igt_subtest("basic-x-tiled-legacy") {
+			igt_require(gem_available_fences(fd) > 0);
 			f.handles[0] = tiled_x_bo;
 
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+			do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 			f.fb_id = 0;
 		}
 
+		igt_describe("Check if addfb2 call works for x and y tiling");
 		igt_subtest("framebuffer-vs-set-tiling") {
+			igt_require(gem_available_fences(fd) > 0);
 			f.handles[0] = gem_bo;
 
 			gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4);
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4) == -EBUSY);
-			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_Y, 1024*4) == -EBUSY);
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+			do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+			igt_assert_eq(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4), -EBUSY);
+			igt_assert_eq(__gem_set_tiling(fd, gem_bo, I915_TILING_Y, 1024*4), -EBUSY);
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 			f.fb_id = 0;
 		}
 
-		f.pitches[0] = 512*4;
+		igt_describe("Test that addfb2 call fails correctly for pitches mismatch");
+			f.pitches[0] = 512*4;
 		igt_subtest("tile-pitch-mismatch") {
+			igt_require(gem_available_fences(fd) > 0);
 			f.handles[0] = tiled_x_bo;
 
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-				   errno == EINVAL);
+			do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		}
 
+		igt_describe("Test that addfb2 call fails correctly for basic y-tiling test");
 		f.pitches[0] = 1024*4;
-		igt_subtest("basic-Y-tiled") {
+		igt_subtest("basic-y-tiled-legacy") {
+			igt_require(!gem_has_lmem(fd));
+			igt_require(gem_available_fences(fd) > 0);
 			f.handles[0] = tiled_y_bo;
 
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-				   errno == EINVAL);
+			do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		}
 
 		igt_fixture {
@@ -367,6 +394,9 @@ static void size_tests(int fd)
 	struct drm_mode_fb_cmd2 f = {};
 	struct drm_mode_fb_cmd2 f_16 = {};
 	struct drm_mode_fb_cmd2 f_8 = {};
+	struct drm_mode_fb_cmd2 *framebuffers[] = {&f, &f_16, &f_8};
+	igt_display_t display;
+	int i;
 
 	f.width = 1024;
 	f.height = 1024;
@@ -384,6 +414,8 @@ static void size_tests(int fd)
 	f_8.pitches[0] = 1024*2;
 
 	igt_fixture {
+		igt_display_require(&display, fd);
+
 		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
 			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
@@ -396,28 +428,29 @@ static void size_tests(int fd)
 	f_16.handles[0] = gem_bo;
 	f_8.handles[0] = gem_bo;
 
+	igt_describe("Check if addfb2 call works with max size of buffer object");
 	igt_subtest("size-max") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
-		f.fb_id = 0;
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_16) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f_16.fb_id) == 0);
-		f.fb_id = 0;
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_8) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f_8.fb_id) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 		f.fb_id = 0;
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_16);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f_16.fb_id);
+		f_16.fb_id = 0;
+		if (igt_display_has_format_mod(&display, DRM_FORMAT_C8, 0)) {
+			do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_8);
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f_8.fb_id);
+			f_8.fb_id = 0;
+		}
 	}
 
 	f.width++;
 	f_16.width++;
 	f_8.width++;
+	igt_describe("Test that addfb2 call fails correctly with increased width of fb");
 	igt_subtest("too-wide") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_16) == -1 &&
-			   errno == EINVAL);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_8) == -1 &&
-			   errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f_16, EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f_8, EINVAL);
 	}
 	f.width--;
 	f_16.width--;
@@ -425,34 +458,46 @@ static void size_tests(int fd)
 	f.height++;
 	f_16.height++;
 	f_8.height++;
+	igt_describe("Test that addfb2 call fails correctly with increased height of fb");
 	igt_subtest("too-high") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_16) == -1 &&
-			   errno == EINVAL);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_8) == -1 &&
-			   errno == EINVAL);
+		for (i = 0; i < ARRAY_SIZE(framebuffers); i++) {
+			igt_debug("Checking framebuffer %i\n", i);
+			igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, framebuffers[i]), -1);
+			if (is_nouveau_device(fd))
+				igt_assert_eq(errno, ERANGE);
+			else
+				igt_assert_eq(errno, EINVAL);
+			errno = 0;
+		}
 	}
 
+	igt_describe("Test that addfb2 call fails correctly with small size of buffer object");
 	f.handles[0] = gem_bo_small;
 	igt_subtest("bo-too-small") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f), -1);
+		igt_assert(errno != 0);
+		if (is_nouveau_device(fd))
+			igt_assert_eq(errno, ERANGE);
+		else
+			igt_assert_eq(errno, EINVAL);
+		errno = 0;
 	}
 
 	/* Just to check that the parameters would work. */
+	igt_describe("Check if addfb2 call works for given height");
 	f.height = 1020;
 	igt_subtest("small-bo") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 		f.fb_id = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly with small buffer object after changing tile");
 	igt_subtest("bo-too-small-due-to-tiling") {
 		igt_require_intel(fd);
+		igt_require(gem_available_fences(fd) > 0);
 		gem_set_tiling(fd, gem_bo_small, I915_TILING_X, 1024*4);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 	}
 
 
@@ -464,7 +509,7 @@ static void size_tests(int fd)
 
 static void addfb25_tests(int fd)
 {
-	struct local_drm_mode_fb_cmd2 f = {};
+	struct drm_mode_fb_cmd2 f = {};
 
 	igt_fixture {
 		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
@@ -477,53 +522,59 @@ static void addfb25_tests(int fd)
 		f.height = 1024;
 		f.pixel_format = DRM_FORMAT_XRGB8888;
 		f.pitches[0] = 1024*4;
-		f.modifier[0] = LOCAL_DRM_FORMAT_MOD_NONE;
+		f.modifier[0] = DRM_FORMAT_MOD_LINEAR;
 
 		f.handles[0] = gem_bo;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for x-tiling with given modifier");
 	igt_subtest("addfb25-modifier-no-flag") {
 		igt_require_fb_modifiers(fd);
 
-		f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
+		f.modifier[0] = I915_FORMAT_MOD_X_TILED;
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 	}
 
 	igt_fixture
-		f.flags = LOCAL_DRM_MODE_FB_MODIFIERS;
+		f.flags = DRM_MODE_FB_MODIFIERS;
 
+	igt_describe("Test that addfb2 call fails correctly for irrelevant modifier");
 	igt_subtest("addfb25-bad-modifier") {
 		igt_require_fb_modifiers(fd);
 
 		f.modifier[0] = ~0;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 	}
 
 	igt_subtest_group {
 		igt_fixture {
 			igt_require_intel(fd);
+			igt_require(gem_available_fences(fd) > 0);
 			gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4);
 			igt_require_fb_modifiers(fd);
 		}
 
-		igt_subtest("addfb25-X-tiled-mismatch") {
-			f.modifier[0] = LOCAL_DRM_FORMAT_MOD_NONE;
-			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
+		igt_describe("Test that addfb2 call fails correctly for irrelevant x-tiling");
+		igt_subtest("addfb25-x-tiled-mismatch-legacy") {
+			f.modifier[0] = DRM_FORMAT_MOD_LINEAR;
+			do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		}
 
-		igt_subtest("addfb25-X-tiled") {
-			f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
-			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		igt_describe("Check if addfb2 call works for x-tiling");
+		igt_subtest("addfb25-x-tiled-legacy") {
+			f.modifier[0] = I915_FORMAT_MOD_X_TILED;
+			do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 			f.fb_id = 0;
 		}
 
+		igt_describe("Check if addfb2 call works for relevant combination of tiling and fbs");
 		igt_subtest("addfb25-framebuffer-vs-set-tiling") {
-			f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
-			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4) == -EBUSY);
-			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_Y, 1024*4) == -EBUSY);
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+			f.modifier[0] = I915_FORMAT_MOD_X_TILED;
+			do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
+			igt_assert_eq(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4), -EBUSY);
+			igt_assert_eq(__gem_set_tiling(fd, gem_bo, I915_TILING_Y, 1024*4), -EBUSY);
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 			f.fb_id = 0;
 		}
 	}
@@ -531,26 +582,20 @@ static void addfb25_tests(int fd)
 		gem_close(fd, gem_bo);
 }
 
-static int addfb_expected_ret(int fd, uint64_t modifier)
+static int addfb_expected_ret(igt_display_t *display, struct drm_mode_fb_cmd2 *f)
 {
-	int gen;
-
-	if (!is_i915_device(fd))
-		return 0;
-
-	gen = intel_gen(intel_get_drm_devid(fd));
-
-	if (modifier == LOCAL_I915_FORMAT_MOD_Yf_TILED)
-		return gen >= 9 && gen < 12 ? 0 : -1;
-	return gen >= 9 ? 0 : -1;
+	return igt_display_has_format_mod(display, f->pixel_format,
+					  f->modifier[0]) ? 0 : -1;
 }
 
 static void addfb25_ytile(int fd)
 {
-	struct local_drm_mode_fb_cmd2 f = {};
-	int gen;
+	struct drm_mode_fb_cmd2 f = {};
+	igt_display_t display;
 
 	igt_fixture {
+		igt_display_require(&display, fd);
+
 		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
 			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
@@ -564,50 +609,99 @@ static void addfb25_ytile(int fd)
 		f.height = 1024;
 		f.pixel_format = DRM_FORMAT_XRGB8888;
 		f.pitches[0] = 1024*4;
-		f.flags = LOCAL_DRM_MODE_FB_MODIFIERS;
-		f.modifier[0] = LOCAL_DRM_FORMAT_MOD_NONE;
+		f.flags = DRM_MODE_FB_MODIFIERS;
+		f.modifier[0] = DRM_FORMAT_MOD_LINEAR;
 
 		f.handles[0] = gem_bo;
 	}
 
-	igt_subtest("addfb25-Y-tiled") {
+	igt_describe("Check if addfb2 call works for y-tiling");
+	igt_subtest("addfb25-y-tiled-legacy") {
 		igt_require_fb_modifiers(fd);
+		igt_require_intel(fd);
 
-		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Y_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) ==
-			   addfb_expected_ret(fd, f.modifier[0]));
-		if (!addfb_expected_ret(fd, f.modifier[0]))
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		f.modifier[0] = I915_FORMAT_MOD_Y_TILED;
+		igt_assert_eq(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f),
+			      addfb_expected_ret(&display, &f));
+		if (!addfb_expected_ret(&display, &f))
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 		f.fb_id = 0;
 	}
 
-	igt_subtest("addfb25-Yf-tiled") {
+	igt_describe("Check if addfb2 call works for yf-tiling");
+	igt_subtest("addfb25-yf-tiled-legacy") {
 		igt_require_fb_modifiers(fd);
+		igt_require_intel(fd);
 
-		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Yf_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) ==
-			   addfb_expected_ret(fd, f.modifier[0]));
-		if (!addfb_expected_ret(fd, f.modifier[0]))
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		f.modifier[0] = I915_FORMAT_MOD_Yf_TILED;
+		igt_assert_eq(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f),
+			      addfb_expected_ret(&display, &f));
+		if (!addfb_expected_ret(&display, &f))
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 		f.fb_id = 0;
 	}
 
-	igt_subtest("addfb25-Y-tiled-small") {
+	igt_describe("Test that addfb2 call fails correctly for y-tiling with given height and modifier");
+	igt_subtest("addfb25-y-tiled-small-legacy") {
 		igt_require_fb_modifiers(fd);
+		igt_require_intel(fd);
 
-		gen = intel_gen(intel_get_drm_devid(fd));
-		igt_require(gen >= 9);
-
-		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Y_TILED;
+		f.modifier[0] = I915_FORMAT_MOD_Y_TILED;
 		f.height = 1023;
 		f.handles[0] = gem_bo_small;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
+		igt_require(addfb_expected_ret(&display, &f) == 0);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_ADDFB2, &f, EINVAL);
 		f.fb_id = 0;
 	}
 
 	igt_fixture {
 		gem_close(fd, gem_bo);
 		gem_close(fd, gem_bo_small);
+		igt_display_fini(&display);
+	}
+}
+
+static void addfb25_4tile(int fd)
+{
+	struct drm_mode_fb_cmd2 f = {};
+	igt_display_t display;
+
+	igt_fixture {
+		igt_display_require(&display, fd);
+
+		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+				DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
+		igt_assert(gem_bo);
+
+		memset(&f, 0, sizeof(f));
+
+		f.width = 1024;
+		f.height = 1024;
+		f.pixel_format = DRM_FORMAT_XRGB8888;
+		f.pitches[0] = 1024*4;
+		f.flags = DRM_MODE_FB_MODIFIERS;
+		f.modifier[0] = DRM_FORMAT_MOD_LINEAR;
+
+		f.handles[0] = gem_bo;
+
+	}
+
+	igt_describe("Check if addfb2 call works for tiling-4");
+	igt_subtest("addfb25-4-tiled") {
+		igt_require_fb_modifiers(fd);
+		igt_require_intel(fd);
+
+		f.modifier[0] = I915_FORMAT_MOD_4_TILED;
+		igt_assert_eq(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f),
+			      addfb_expected_ret(&display, &f));
+		if (!addfb_expected_ret(&display, &f))
+			do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
+		f.fb_id = 0;
+	}
+
+	igt_fixture {
+		gem_close(fd, gem_bo);
+		igt_display_fini(&display);
 	}
 }
 
@@ -630,7 +724,7 @@ static void prop_tests(int fd)
 
 		f.handles[0] = gem_bo;
 
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
 	}
 
 	get_props.props_ptr = (uintptr_t) &prop;
@@ -638,40 +732,40 @@ static void prop_tests(int fd)
 	get_props.count_props = 1;
 	get_props.obj_id = f.fb_id;
 
+	igt_describe("Test that get-properties ioctl call fails correctly for invalid object type");
 	igt_subtest("invalid-get-prop-any") {
 		get_props.obj_type = 0; /* DRM_MODE_OBJECT_ANY */
 
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_GETPROPERTIES,
-				    &get_props) == -1 && errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_OBJ_GETPROPERTIES, &get_props, EINVAL);
 	}
 
+	igt_describe("Test that get-properties ioctl call fails correctly for fb mode object");
 	igt_subtest("invalid-get-prop") {
 		get_props.obj_type = DRM_MODE_OBJECT_FB;
 
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_GETPROPERTIES,
-				    &get_props) == -1 && errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_OBJ_GETPROPERTIES, &get_props, EINVAL);
 	}
 
 	set_prop.value = 0;
 	set_prop.prop_id = 1;
 	set_prop.obj_id = f.fb_id;
 
+	igt_describe("Test that set-properties ioctl call fails correctly for invalid object type");
 	igt_subtest("invalid-set-prop-any") {
 		set_prop.obj_type = 0; /* DRM_MODE_OBJECT_ANY */
 
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_SETPROPERTY,
-				    &set_prop) == -1 && errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_OBJ_SETPROPERTY, &set_prop, EINVAL);
 	}
 
+	igt_describe("Test that get-properties ioctl call fails correctly for fb mode object");
 	igt_subtest("invalid-set-prop") {
 		set_prop.obj_type = DRM_MODE_OBJECT_FB;
 
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_SETPROPERTY,
-				    &set_prop) == -1 && errno == EINVAL);
+		do_ioctl_err(fd, DRM_IOCTL_MODE_OBJ_SETPROPERTY, &set_prop, EINVAL);
 	}
 
 	igt_fixture
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 
 }
 
@@ -691,9 +785,10 @@ static void master_tests(int fd)
 
 		f.handles[0] = gem_bo;
 
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
 	}
 
+	igt_describe("Check that only master can rmfb");
 	igt_subtest("master-rmfb") {
 		int master2_fd;
 
@@ -701,7 +796,7 @@ static void master_tests(int fd)
 
 		master2_fd = drm_open_driver_master(DRIVER_ANY);
 
-		igt_assert_eq(rmfb(master2_fd, f.fb_id), -ENOENT);
+		do_ioctl_err(master2_fd, DRM_IOCTL_MODE_RMFB, &f.fb_id, ENOENT);
 
 		igt_device_drop_master(master2_fd);
 		close(master2_fd);
@@ -710,13 +805,13 @@ static void master_tests(int fd)
 	}
 
 	igt_fixture
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+		do_ioctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id);
 
 }
 
 static bool has_addfb2_iface(int fd)
 {
-	struct local_drm_mode_fb_cmd2 f = {};
+	struct drm_mode_fb_cmd2 f = {};
 	int err;
 
 	err = 0;
@@ -756,6 +851,8 @@ igt_main
 
 	addfb25_ytile(fd);
 
+	addfb25_4tile(fd);
+
 	tiling_tests(fd);
 
 	prop_tests(fd);
